/******************************************************************************
*
* Arena 3D: For the 89, 92+, and v200
* By Malcolm Smith
* m@malcolm-s.net
* Started on August 20th, 2002
* Version v0.65 - with Titanium support
*
******************************************************************************/

/*
Quitting values:
1. ESC Pressed
2. End of Level

-1. Out of Memory
-2. GFX File Error
-3. Map error
*/

#define USE_TI89
#define USE_TI92PLUS
#define USE_V200

#include "includes.h"		//Include everything


void
_main (void)
{
  LCD_BUFFER lcd;
  short result;
  FATCONFIG fc;


  //-------------------------------------------------------------------------
  // before we try to do anything else we have to initialize the FAT engine
  // its only valid to proceed if FAT_LOADOKAY gets returned
  //-------------------------------------------------------------------------
  if ((result = FAT_LoadEngine ()) != FAT_LOADOKAY)
    {
      ShowFatLoadError ((result < 0 && result >= -4) ? -result : 0);
      return;
    }

  //Set up the FATCONFIG structure
  SetupFC (fc);
  //Try to open Arena 3D's GFX files
  if (!OpenGraphicsFiles ())
    return;
  //Save the LCD screen
  memcpy (lcd, LCD_MEM, LCD_SIZE);
  //Initialize the global game structure
  InitGameStruct ();
  //Load the Arena config file, if it exists
  LoadConfig ();
  //Check if game parameters were passed to Arena
  if (!CheckForGameParameters ())
    return;
  //Try to allocate all the memory Arena needs
  //(sprite configs, and HUD space)
  if (!AllocateMemory ())
    return;
  //If it worked, set the FATCONFIG's sprites to the new memory
  fc.sprites = allspr;
  //Set up the player sprite pointers
  SetupPlayerPointers ();
  //Redirect Auto-Interrupts for grayscale and _rowread keyboard functions
  InstallDummyHandlerForINTs ();
  //Set the global quitting flag to 0, so things will run
  gamedata.quitting = 0;

  //-------------------------------------------------------------------------
  // initialize grayscales, clear both planes and set the destination plane
  // addresses for the FAT-Engine
  //-------------------------------------------------------------------------
  if (!GrayOn ())
    {
      RestoreHandlerForINTs ();
      FAT_UnloadEngine ();
      FreeMemory ();
      ArenaShowError (ARENA_OUT_OF_MEM);
      return;
    }

  //-------------------------------------------------------------------------
  // display the FAT-Engine's splashscreen
  //-------------------------------------------------------------------------
  if (gamedata.cfg.fatlogo && noskip)
    FAT_DisplayLogo (30);

  fc.dest_plane0 = (void*)((BYTE*)GetPlane (0) + 30 * 2);	// center vertically
  fc.dest_plane1 = (void*)((BYTE*)GetPlane (1) + 30 * 2);	// center vertically

  //-------------------------------------------------------------------------
  // setup calculator type specials
  //-------------------------------------------------------------------------
  if (TI89)
    {
      display_offsetx = 0;
      display_offsety = 0;
    }
  else
    {
      // TI92p or V200
      display_offsetx = 32;
      display_offsety = 14;
      fc.dest_plane0 += (14 * 30 + 4) / 2;
      fc.dest_plane1 += (14 * 30 + 4) / 2;
    }
  health.x += display_offsetx;
  if (noskip)
    ShowLogo ();

  while (1)
    {
      if (gamedata.quitting < 0)
	break;
      if (!HandleMenu (&fc))
	break;
      if (gamedata.quitting < 0)
	break;	
      if (noskip)
	SetupGame ();
      if (gamedata.quitting < 0)
	break;

      //-------------------------------------------------------------------------
      // set predraw callback function which is called before the vertical strips
      // get rendered, but after the DEPTHDATA structure is filled ...
      //-------------------------------------------------------------------------
      FAT_SetPreHUDCallback (PreHUDCallback);
      //For an inverted screen when the player is hit

      //-------------------------------------------------------------------------
      // initialize the GUI
      //-------------------------------------------------------------------------

      if (!gamedata.quitting)
	{
	  BlackGrayScreen ();
	  InitGUI ();
	}

      FAT_SetHUD (HUDlight, HUDdark, HUDmask);

      SetupHUD (GENIDX_hudRPG);

      int5_counter = 0;
      count5_start = int5_counter;

      GenerateRespawnPointsFromMap (&fc);
      //Set up gametype-specific stuff, if applicable
      if (gamedata.gametype == CTF)
	SetupCTF (&fc);
      if (gamedata.gametype == DM)
	FixTeams ();
      SpawnAllPlayers (&fc);

      //-------------------------------------------------------------------------
      // the main rendering loop ...
      //-------------------------------------------------------------------------
      while (!gamedata.quitting)
	{
	  if (gamedata.instagib)
	    {
	      while (RocketsHandle (&fc));
	    }
	  else
	    RocketsHandle (&fc);
	  PlayersHandle (&fc);
	  switch (gamedata.gametype)
	    {
	    case CTF:
	      CTFhandle ();
	      break;
	    }
	  fc.nr_sprites =
	    gamedata.sprites.rockets + gamedata.sprites.players +
	    gamedata.sprites.ctfflags;

	  FAT_Render (&fc);
/*
{
	char buf[30]; 
	sprintf(buf,"%i %i %i    ",
gamedata.player[0].desire,gamedata.player[2].desire,gamedata.player[4].desire
					);
	DrawGrayStrExt(100,60,buf,F_4x6,A_REVERSE);
}
*/

	  DrawHealthAndScore ();
	  DrawCompass (&fc);
	  DrawMultiplier ();

	  if (!HandlePlayerMovement (&fc))
	    break;

	}

      if (gamedata.quitting == 2)
	ShowEndOfMatch ();
      if (gamedata.quitting > 0)
	gamedata.quitting = 0;
    }

  //-------------------------------------------------------------------------
  // some clean stuff ...
  //-------------------------------------------------------------------------
  FAT_AllKeysReleased ();
  dissolveToLCD (lcd);
  GrayOff ();
  SaveConfig ();
  FreeMemory ();
  ShowFinalErrors ();
  FAT_UnloadEngine ();
  RestoreHandlerForINTs ();
  OSqclear (kbd_queue ());

}
