// C Header File
// Created 8/24/2002; 12:45:42 PM

void DisplayCurrentScores (void);

//=============================================================================
// returns 0 if move would bring the character too near to a wall
//=============================================================================
short
ValidMove (FATCONFIG * fc, short x, short y)
{
#define DIST_FROM_WALL 15

  // if we would come too near to a solid square...
  if ((*
       (fc->map_data + ((y + DIST_FROM_WALL) >> 6) * fc->map_width +
	((x + DIST_FROM_WALL) >> 6)) > 0)
      ||
      (*
       (fc->map_data + ((y - DIST_FROM_WALL) >> 6) * fc->map_width +
	((x - DIST_FROM_WALL) >> 6)) > 0)
      ||
      (*
       (fc->map_data + ((y + DIST_FROM_WALL) >> 6) * fc->map_width +
	((x - DIST_FROM_WALL) >> 6)) > 0)
      ||
      (*
       (fc->map_data + ((y - DIST_FROM_WALL) >> 6) * fc->map_width +
	((x + DIST_FROM_WALL) >> 6)) > 0))
    {
      return 0;			// false
    }
  return 1;			// true
}


//=============================================================================
// move player forward or backward depending on player's angle ...
//
// special player move routine (takes doors into account) AND don't allows
// the user to come too near to a wall, but try to slide automatically.
//=============================================================================
void
PlayerMove (FATCONFIG * fc, short forward_right, short strafe)
{
  short tmp = (strafe) ? +ANGLE_090 : 0;
  short stepx = FAT_Cos16384 (fc->cam_orientation + tmp) >> 11;
  short stepy = FAT_Sin16384 (fc->cam_orientation + tmp) >> 11;
  short newxpos = fc->cam_xpos + ((forward_right) ? stepx : -stepx);
  short newypos = fc->cam_ypos + ((forward_right) ? stepy : -stepy);
  short newxmod = newxpos >> 6;
  short newymod = newypos >> 6;

  //-------------------------------------------------------------------------
  // check if we are still within the map
  //-------------------------------------------------------------------------
  if (newxmod > fc->map_width - 1 ||
      newymod > fc->map_height - 1 || newxmod < 0 || newymod < 0)
    {
      return;
    }

  if (!ValidMove (fc, newxpos, newypos))
    {				// if new position is invalid ...
      // ...but old position is OK, we're against a wall to east or west
      if (ValidMove (fc, fc->cam_xpos, newypos))
	{
	  // don't change angle if movement is perpendicular to the wall
	  if (stepy != 0)
	    {
	      // if sign(stepx)==sign(stepy), rotate clockwise = turn right
	      if ((stepx > 0) == (stepy > 0))
		PlayerTurn (fc, 1, ANGLE_015 / 2);
	      else
		PlayerTurn (fc, 0, ANGLE_015 / 2);
	      // still allow movement along y = parallel to wall
	      fc->cam_ypos = newypos;
	    }
	}
      // ...but (new x, old y) is OK, we're against a wall to north or south
      if (ValidMove (fc, newxpos, fc->cam_ypos))
	{
	  // don't change angle if movement is perpendicular to the wall
	  if (stepx != 0)
	    {
	      // if sign(stepx)==sign(stepy), rotate counterclockwise = turn left
	      if ((stepx > 0) == (stepy > 0))
		PlayerTurn (fc, 0, ANGLE_015 / 2);
	      else
		PlayerTurn (fc, 1, ANGLE_015 / 2);
	      // still allow movement along x = parallel to wall
	      fc->cam_xpos = newxpos;
	    }
	}
      // ...otherwise we're in a corner, so do nothing
      return;
    }

  //-------------------------------------------------------------------------
  // everything okay -> let's set the new position
  //-------------------------------------------------------------------------
  fc->cam_xpos = newxpos;
  fc->cam_ypos = newypos;
}

short
HandlePlayerMovement (FATCONFIG * fc)
{
  PADSTATES key = 0;
  short moving_speed;
  short turning_speed;

  gamedata.player[REAL_PLAYER_IDX].spr->xpos = fc->cam_xpos;
  gamedata.player[REAL_PLAYER_IDX].spr->ypos = fc->cam_ypos;
  gamedata.player[REAL_PLAYER_IDX].spr->orientation = fc->cam_orientation;

  key = FAT_GetPadState ();
  if (key & PADSTATE_ESC)
    return 0;
  if (key & PADSTATE_APPS)
    {
      DisplayCurrentScores ();
    }
  if (_keytest (gamedata.cfg.fire.row, gamedata.cfg.fire.key))
    {
      SpawnRocket (fc, REAL_PLAYER_IDX);
    }
  if (KeyCheck (hitfind))
    {
      fc->cam_orientation = gamedata.player[REAL_PLAYER_IDX].last_hit_dir;
    }
  if (KeyCheck (speedup))
    {
      gamedata.speed = 2;
    }
  else
    gamedata.speed = 1;

  if (key & PADSTATE_UPDOWN)
    {
      for (moving_speed = 0; moving_speed < MOVE_SPEED; moving_speed++)
	PlayerMove (fc, key & PADSTATE_UP, 0);
    }
  if (key & PADSTATE_LEFTRIGHT)
    {
      if (KeyCheck (strafe))
	{			//Strafe
	  for (moving_speed = 0; moving_speed < MOVE_SPEED; moving_speed++)
	    PlayerMove (fc, key & PADSTATE_RIGHT, 1);
	}
      else
	{			//Turn
	  for (turning_speed = 0; turning_speed < TURN_SPEED; turning_speed++)
	    PlayerTurn (fc, key & PADSTATE_RIGHT, ANGLE_015 / 2);
	}
    }

  return 1;			//Esc wasn't pressed
}
